local HeadCount = LibStub("AceAddon-3.0"):GetAddon("HeadCount2")
local L = LibStub("AceLocale-3.0"):GetLocale("HeadCount2")

local Fields = {
	raidList = {},
	numberOfRaids = 0
}

local RaidListWrapper = HeadCount:NewClassFunctions()

function RaidListWrapper:Init(args)
	if args then
		raidList = args["raidList"] or {}
		numberOfRaids = args["numberOfRaids"] or 0
	end
end

-- Retrieves the most recent raid.
-- @return object Returns the most recent raid or nil if none exists.
function RaidListWrapper:retrieveMostRecentRaid()
	local raid = nil
	local orderedRaidList = self:retrieveOrderedRaidList(true)
	local numberOfRaids = #orderedRaidList
	if (numberOfRaids > 0) then
		raid = orderedRaidList[numberOfRaids]
	end
	return raid
end

function RaidListWrapper:retrieveMostRecentRaidId()
	local raidId = 0
	local raid = self:retrieveMostRecentRaid()
	if (raid) then
		raidId = raid:retrieveStartingTime().epoc
	end
	return raidId
end

-- Retrieves an ordered raid list.
-- @param isDescending The sort order.
-- @return table Returns an ordered raid list.
function RaidListWrapper:retrieveOrderedRaidList(isDescending)
	local orderedRaidList = {}

	for k,v in pairs(self.raidList) do
		table.insert(orderedRaidList, v)
	end
	table.sort(orderedRaidList, function(a, b)
		local aKey = a:retrieveStartingTime().epoc
		local bKey = b:retrieveStartingTime().epoc
		if isDescending then
			return aKey < bKey
		else
			return aKey > bKey
		end
	end)
	return orderedRaidList
end

-- Adds a raid.
-- @param raid The raid.
function RaidListWrapper:addRaid(id, raid)
	if id and raid then
		self.raidList[id] = raid
		self.numberOfRaids = self.numberOfRaids + 1
	end
end

-- Remove a raid.
-- @param id The raid id.
-- @return boolean Returns true if the raid was removed and false otherwise.
function RaidListWrapper:removeRaid(id)
	local isRemoved = false
	if self.raidList[id] then
		-- the raid exists
		self.raidList[id] = nil
		self.numberOfRaids = self.numberOfRaids - 1
		isRemoved = true
	end
	return isRemoved
end

-- Remove all raids
function RaidListWrapper:removeAll()
	table.wipe(self.raidList)
	self.numberOfRaids = 0
end

-- Remove old raids
-- @param number The pruning time, in number of weeks
function RaidListWrapper:pruneRaids(pruningTimeInWeeks)
	local pruningTimeType = type(pruningTimeInWeeks)

	if pruningTimeType == "number" and pruningTimeInWeeks > 0 then
		-- parameter is a valid number
		-- parameter is a number greater than zero (1 or more weeks)
		local utcDateTimeInSeconds = HeadCount:getUTCDateTimeInSeconds()		-- the current date and time
		local activityTime = HeadCount:NewInstance("Time",utcDateTimeInSeconds)

		local pruningTime = -1 * pruningTimeInWeeks	-- multiply by -1 to look back in time
		activityTime:AddWeeks(pruningTime)

		HeadCount:LogDebug(string.format(L["debug.raid.prune.date"], HeadCount.TITLE, HeadCount.VERSION, HeadCount:getDateTimeAsString(activityTime)))

		for k,v in pairs(self.raidList) do
			local endingTime = v:retrieveEndingTime()
			local isFinalized = v.isFinalized
			if endingTime and isFinalized then
				-- raid has an ending time and is finalized, candidate for automatical deletion
				local endingTimeInSeconds = endingTime.epoc
				local activityTimeInSeconds = activityTime.epoc
				if endingTimeInSeconds <= activityTimeInSeconds then
					local startingTime = v:retrieveStartingTime()
					HeadCount:LogDebug(string.format(L["debug.raid.prune.delete"], HeadCount.TITLE, HeadCount.VERSION, HeadCount:getDateTimeAsString(startingTime), HeadCount:getDateTimeAsString(endingTime)))

					local startingTimeInSeconds = startingTime.epoc
					self:removeRaid(startingTimeInSeconds)	-- remove the raid
				end
			end
		end
	end
end

-- Determines if a raid is present.
-- @param id The raid id.
-- @return boolean Returns true if the raid is present and false otherwise.
function RaidListWrapper:isRaidPresent(id)
	local isPresent = false
	if (self.raidList[id]) then
		isPresent = true
	end
	return isPresent
end

-- Gets a raid by its id.
-- @param id The raid id.
-- @return object Returns the raid if it exists or nil otherwise.
function RaidListWrapper:getRaidById(id)
	return self.raidList[id]
end


HeadCount:DefineClass("RaidListWrapper",Fields,RaidListWrapper)